"""Top-level package for lotecc."""
from lotecc.lotecc import lote_chinese_conversion

__all__ = ['lote_chinese_conversion']

__author__ = """Allen Shaw"""
__email__ = 'lonsty@sina.com'
__version__ = '0.1.1'
